//##protect##"FileHeader"
/** @file PCIe1516.cs
 *
 *  @authors S. Beuque, C. Guinot, J. Krauth, N. Reibel, J. Rose, L. Sartory, S. Weber
 *  @date    Creation: 27.11.2012
 *  @date    $LastChangedDate: 2012-12-07 10:36:45 +0100 (Fr, 07. Dez 2012) $
 *  @version $LastChangedRevision: 3640 $
 *
 *  @par DESCRIPTION
 *  PCIe1516 .NET interface functions
 *
 *  @par LICENCE
 *  @verbatim
 *  Copyright (C) 2009 - 2012 ADDI-DATA GmbH
 *
 *  ADDI-DATA GmbH
 *  Airpark Business Center
 *  Airport Boulevard B210
 *  77836 Rheinmnster
 *  Germany
 *  Tel: +49(0)7229/1847-0    |  Fax: +49(0)7229/1847-200
 *  http://www.addi-data-com  |  info@addi-data.com
 *  @endverbatim
 **/
//##protect##"FileHeader"

using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace PCIe1516_NET
{
#region "User declarations"
//##protect##"UserDeclarations"
//##protect##"UserDeclarations"
#endregion

    public class PCIe1516
    {
        private int index;
        private IntPtr handle;

		private DllImport.InterruptFunctionEventHandler interruptFunctionHandler;

#region "Event Args and Handlers"
        private DigitalOutputEventArgs digitalOutputEventArgs;
        public delegate void DigitalOutputEventHandler(object source, DigitalOutputEventArgs eventArgs);
        public event DigitalOutputEventHandler DigitalOutputEventReceived;

        public delegate void WatchdogEventHandler(object source);
        public event WatchdogEventHandler WatchdogEventReceived;

//##protect##"EventArgsAndHandlersDeclaration"
//##protect##"EventArgsAndHandlersDeclaration"
#endregion
#region "Constructor/Destructor"
        /// <summary>
        /// Class constructor
        /// </summary>
        private PCIe1516()
        {
            this.index = -1;
            this.handle = IntPtr.Zero;

            // Create the event args (once by board)
            this.digitalOutputEventArgs = new DigitalOutputEventArgs();

            // Create the interrupt handler (once by board)
            this.interruptFunctionHandler = new DllImport.InterruptFunctionEventHandler(InterruptFunction);

            // Tell the garbage collector to keep the pointer
            GC.KeepAlive(this.interruptFunctionHandler);

//##protect##"Constructor"
//##protect##"Constructor"
        }

        /// <summary>
        /// Class destructor
        /// </summary>
        ~PCIe1516()
        {
            DllImport.i_PCIe1516_CloseBoard(this.handle);
            this.index = -1;
            this.handle = IntPtr.Zero;

//##protect##"Destructor"
//##protect##"Destructor"
        }
#endregion
#region "Utils"
        /// <summary>
        /// Creates an instance of a PCIe1516 board
        /// </summary>
        /// <param name="index">The index of the board you want to open</param>
        /// <returns>null on error, instance of PCIe1516 on success</returns>
        public static PCIe1516 OpenBoard(int index)
        {
            PCIe1516 boardInstance;
            int nbBoard;
            IntPtr handle = IntPtr.Zero;

            // Get the number of boards
			nbBoard = GetBoardCount();
            if (nbBoard == -1)
            {
                Console.WriteLine("GetBoardCount failed!");
                return null;
            }

            // Check that the index is in the range
            if (index < 0 || index > nbBoard)
            {
                Console.WriteLine("Invalid index!");
                return null;
            }

            // Try to open the board
            if (DllImport.i_PCIe1516_OpenBoardViaIndex((Byte)index, out handle) != 0)
            {
                Console.WriteLine("i_PCIe1516_OpenBoardViaIndex failed!");
                return null;
            }

            // Create a new instance of the board
            boardInstance = new PCIe1516();

            // Set its parameters
            boardInstance.index = index;
            boardInstance.handle = handle;

            // Return the instance
            return boardInstance;
        }

        /// <summary>
        /// Returns the number of PCIe1516 boards plugged in the computer
        /// </summary>
        /// <returns>The number of boards</returns>
        public static int GetBoardCount()
        {
            int ret;
            byte nbBoard = 0;

            // Call the DLL function
            ret = DllImport.i_PCIe1516_GetNumberOfBoards(out nbBoard);
            if (ret != 0)
				return -1;

             // Success
            return (int)nbBoard;
        }

        /// <summary>
        /// Returns the description of the last error that occurred
        /// </summary>
        /// <returns>A string describing the last error that occurred</returns>
        public string GetLastError()
        {
            // Call the DLL function
            return Marshal.PtrToStringAnsi(DllImport.pc_PCIe1516_GetLastError(this.handle));
        }
#endregion
#region "Interrupt handling"
        /// <summary>
        /// Signals to all the listeners that an DigitalOutput event occurred.
        /// </summary>
        /// <param name="eventArgs">The arguments of the event</param>
        protected virtual void OnDigitalOutputEventReceived(DigitalOutputEventArgs eventArgs)
		{
            if (this.DigitalOutputEventReceived != null)
                this.DigitalOutputEventReceived(this, eventArgs);
        }

        /// <summary>
        /// Signals to all the listeners that an Watchdog event occurred.
        /// </summary>
        protected virtual void OnWatchdogEventReceived()
		{
            if (this.WatchdogEventReceived != null)
                this.WatchdogEventReceived(this);
        }

        /// <summary>
        /// Interrupt function - Called each time the board has generated an interrupt
        /// </summary>
        private void InterruptFunction(IntPtr handle, Byte overflow, UInt32 source, UInt32 mask, IntPtr args, UInt32 argsSize)
        {
            // DigitalOutput ?
            if ((source & 0x1) != 0)
            {
				this.digitalOutputEventArgs.interruptMask = mask;

                // Signal the event
                this.OnDigitalOutputEventReceived(this.digitalOutputEventArgs);
            }
            // Watchdog ?
            if ((source & 0x2) != 0)
            {
                // Signal the event
                this.OnWatchdogEventReceived();
            }

//##protect##"interruptFunctionBody"
//##protect##"interruptFunctionBody"
        }

        /// <summary>
        /// Set the board interrupt routine
        /// </summary>
        public int SetBoardIntRoutine()
        {
            return (int)DllImport.i_PCIe1516_SetBoardIntRoutine(this.handle, interruptFunctionHandler);
        }

        /// <summary>
        /// Reset the board interrupt routine
        /// </summary>
        public int ResetBoardIntRoutine()
        {
            return (int)DllImport.i_PCIe1516_ResetBoardIntRoutine(this.handle);
        }

#endregion

#region "PCIe1516"

#endregion
#region "DigitalInput"

		/// <summary>
		/// Reads 1 digital input of the board
		/// </summary>
		/// <param name="deviceHandle">Handle of the board</param>
		/// <param name="channel">Channel to be read</param>
		/// <param name="value">Status of the digital input</param>
		public int Read1DigitalInput(Byte channel, out Byte value)
		{
			int returnValue;
//##protect##"functionStart_Read1DigitalInput"
//##protect##"functionStart_Read1DigitalInput"
			returnValue = DllImport.i_PCIe1516_Read1DigitalInput(this.handle, channel, out value);
//##protect##"functionEnd_Read1DigitalInput"
//##protect##"functionEnd_Read1DigitalInput"
			return returnValue;
		}

		/// <summary>
		/// Reads 4 digital inputs of the board
		/// </summary>
		/// <param name="deviceHandle">Handle of the board</param>
		/// <param name="port">Port to be read. One port includes 4 channels</param>
		/// <param name="value">Status of the digital inputs</param>
		public int Read4DigitalInputs(Byte port, out Byte value)
		{
			int returnValue;
//##protect##"functionStart_Read4DigitalInputs"
//##protect##"functionStart_Read4DigitalInputs"
			returnValue = DllImport.i_PCIe1516_Read4DigitalInputs(this.handle, port, out value);
//##protect##"functionEnd_Read4DigitalInputs"
//##protect##"functionEnd_Read4DigitalInputs"
			return returnValue;
		}

		/// <summary>
		/// Reads 8 digital inputs of the board
		/// </summary>
		/// <param name="deviceHandle">Handle of the board</param>
		/// <param name="value">Status of the digital inputs</param>
		public int Read8DigitalInputs(out Byte value)
		{
			int returnValue;
//##protect##"functionStart_Read8DigitalInputs"
//##protect##"functionStart_Read8DigitalInputs"
			returnValue = DllImport.i_PCIe1516_Read8DigitalInputs(this.handle, out value);
//##protect##"functionEnd_Read8DigitalInputs"
//##protect##"functionEnd_Read8DigitalInputs"
			return returnValue;
		}

#endregion
#region "DigitalOutput"

		/// <summary>
		/// Activates the digital output memory of the board
		/// </summary>
		/// <param name="deviceHandle">Handle of the board</param>
		public int SetDigitalOutputMemoryOn()
		{
			int returnValue;
//##protect##"functionStart_SetDigitalOutputMemoryOn"
//##protect##"functionStart_SetDigitalOutputMemoryOn"
			returnValue = DllImport.i_PCIe1516_SetDigitalOutputMemoryOn(this.handle);
//##protect##"functionEnd_SetDigitalOutputMemoryOn"
//##protect##"functionEnd_SetDigitalOutputMemoryOn"
			return returnValue;
		}

		/// <summary>
		/// Desactivates the digital output memory of the board
		/// </summary>
		/// <param name="deviceHandle">Handle of the board</param>
		public int SetDigitalOutputMemoryOff()
		{
			int returnValue;
//##protect##"functionStart_SetDigitalOutputMemoryOff"
//##protect##"functionStart_SetDigitalOutputMemoryOff"
			returnValue = DllImport.i_PCIe1516_SetDigitalOutputMemoryOff(this.handle);
//##protect##"functionEnd_SetDigitalOutputMemoryOff"
//##protect##"functionEnd_SetDigitalOutputMemoryOff"
			return returnValue;
		}

		/// <summary>
		/// Sets a digital output of the board
		/// </summary>
		/// <param name="deviceHandle">Handle of the board</param>
		/// <param name="channel">Digital output to be set</param>
		public int Set1DigitalOutputOn(Byte channel)
		{
			int returnValue;
//##protect##"functionStart_Set1DigitalOutputOn"
//##protect##"functionStart_Set1DigitalOutputOn"
			returnValue = DllImport.i_PCIe1516_Set1DigitalOutputOn(this.handle, channel);
//##protect##"functionEnd_Set1DigitalOutputOn"
//##protect##"functionEnd_Set1DigitalOutputOn"
			return returnValue;
		}

		/// <summary>
		/// Resets a digital output of the board
		/// </summary>
		/// <param name="deviceHandle">Handle of the board</param>
		/// <param name="channel">Digital output to be reset</param>
		public int Set1DigitalOutputOff(Byte channel)
		{
			int returnValue;
//##protect##"functionStart_Set1DigitalOutputOff"
//##protect##"functionStart_Set1DigitalOutputOff"
			returnValue = DllImport.i_PCIe1516_Set1DigitalOutputOff(this.handle, channel);
//##protect##"functionEnd_Set1DigitalOutputOff"
//##protect##"functionEnd_Set1DigitalOutputOff"
			return returnValue;
		}

		/// <summary>
		/// Sets several digital outputs of the board
		/// </summary>
		/// <param name="deviceHandle">Handle of the board</param>
		/// <param name="port">Port on which digital outputs have to be set</param>
		/// <param name="mask">Digital output mask value to set</param>
		public int Set4DigitalOutputsOn(Byte port, Byte mask)
		{
			int returnValue;
//##protect##"functionStart_Set4DigitalOutputsOn"
//##protect##"functionStart_Set4DigitalOutputsOn"
			returnValue = DllImport.i_PCIe1516_Set4DigitalOutputsOn(this.handle, port, mask);
//##protect##"functionEnd_Set4DigitalOutputsOn"
//##protect##"functionEnd_Set4DigitalOutputsOn"
			return returnValue;
		}

		/// <summary>
		/// Resets several digital outputs of the board
		/// </summary>
		/// <param name="deviceHandle">Handle of the board</param>
		/// <param name="port">Port on which digital outputs have to be reset</param>
		/// <param name="mask">Digital output mask value to reset</param>
		public int Set4DigitalOutputsOff(Byte port, Byte mask)
		{
			int returnValue;
//##protect##"functionStart_Set4DigitalOutputsOff"
//##protect##"functionStart_Set4DigitalOutputsOff"
			returnValue = DllImport.i_PCIe1516_Set4DigitalOutputsOff(this.handle, port, mask);
//##protect##"functionEnd_Set4DigitalOutputsOff"
//##protect##"functionEnd_Set4DigitalOutputsOff"
			return returnValue;
		}

		/// <summary>
		/// Sets several digital outputs of the board
		/// </summary>
		/// <param name="deviceHandle">Handle of the board</param>
		/// <param name="mask">Digital output mask value to set</param>
		public int Set8DigitalOutputsOn(Byte mask)
		{
			int returnValue;
//##protect##"functionStart_Set8DigitalOutputsOn"
//##protect##"functionStart_Set8DigitalOutputsOn"
			returnValue = DllImport.i_PCIe1516_Set8DigitalOutputsOn(this.handle, mask);
//##protect##"functionEnd_Set8DigitalOutputsOn"
//##protect##"functionEnd_Set8DigitalOutputsOn"
			return returnValue;
		}

		/// <summary>
		/// Resets several digital outputs of the board
		/// </summary>
		/// <param name="deviceHandle">Handle of the board</param>
		/// <param name="mask">Digital output mask value to reset</param>
		public int Set8DigitalOutputsOff(Byte mask)
		{
			int returnValue;
//##protect##"functionStart_Set8DigitalOutputsOff"
//##protect##"functionStart_Set8DigitalOutputsOff"
			returnValue = DllImport.i_PCIe1516_Set8DigitalOutputsOff(this.handle, mask);
//##protect##"functionEnd_Set8DigitalOutputsOff"
//##protect##"functionEnd_Set8DigitalOutputsOff"
			return returnValue;
		}

		/// <summary>
		/// Sets / resets several digital outputs of the board without using the output memory
		/// </summary>
		/// <param name="deviceHandle">Handle of the board</param>
		/// <param name="mask">Digital output mask value to set / reset</param>
		public int SetAllDigitalOutputs(Byte mask)
		{
			int returnValue;
//##protect##"functionStart_SetAllDigitalOutputs"
//##protect##"functionStart_SetAllDigitalOutputs"
			returnValue = DllImport.i_PCIe1516_SetAllDigitalOutputs(this.handle, mask);
//##protect##"functionEnd_SetAllDigitalOutputs"
//##protect##"functionEnd_SetAllDigitalOutputs"
			return returnValue;
		}

		/// <summary>
		/// Reads 1 digital output of the board
		/// </summary>
		/// <param name="deviceHandle">Handle of the board</param>
		/// <param name="channel">Channel to be read</param>
		/// <param name="status">Status of the digital output (0: off, 1: on)</param>
		public int Get1DigitalOutputStatus(Byte channel, out Byte status)
		{
			int returnValue;
//##protect##"functionStart_Get1DigitalOutputStatus"
//##protect##"functionStart_Get1DigitalOutputStatus"
			returnValue = DllImport.i_PCIe1516_Get1DigitalOutputStatus(this.handle, channel, out status);
//##protect##"functionEnd_Get1DigitalOutputStatus"
//##protect##"functionEnd_Get1DigitalOutputStatus"
			return returnValue;
		}

		/// <summary>
		/// Reads 4 digital outputs of the board
		/// </summary>
		/// <param name="deviceHandle">Handle of the board</param>
		/// <param name="port">Port on which digital outputs have to be read</param>
		/// <param name="status">Status of the digital outputs (0: off, 1: on)</param>
		public int Get4DigitalOutputStatus(Byte port, out Byte status)
		{
			int returnValue;
//##protect##"functionStart_Get4DigitalOutputStatus"
//##protect##"functionStart_Get4DigitalOutputStatus"
			returnValue = DllImport.i_PCIe1516_Get4DigitalOutputStatus(this.handle, port, out status);
//##protect##"functionEnd_Get4DigitalOutputStatus"
//##protect##"functionEnd_Get4DigitalOutputStatus"
			return returnValue;
		}

		/// <summary>
		/// Read the 8 digital outputs status of the board
		/// </summary>
		/// <param name="deviceHandle">Handle of the board</param>
		/// <param name="status">Status of the digital outputs (0: off, 1: on)</param>
		public int Get8DigitalOutputsStatus(out Byte status)
		{
			int returnValue;
//##protect##"functionStart_Get8DigitalOutputsStatus"
//##protect##"functionStart_Get8DigitalOutputsStatus"
			returnValue = DllImport.i_PCIe1516_Get8DigitalOutputsStatus(this.handle, out status);
//##protect##"functionEnd_Get8DigitalOutputsStatus"
//##protect##"functionEnd_Get8DigitalOutputsStatus"
			return returnValue;
		}

#endregion
#region "Watchdog"

		/// <summary>
		/// Initializes and starts the watchdog of the board
		/// </summary>
		/// <param name="deviceHandle">Handle of the board</param>
		/// <param name="reloadValue">Watchdog reload value</param>
		/// <param name="timeBase">Time base of the watchdog (0 for microseconds, 1 for milliseconds and 2 for seconds)</param>
		public int InitAndStartWatchdog(UInt16 reloadValue, Byte timeBase)
		{
			int returnValue;
//##protect##"functionStart_InitAndStartWatchdog"
//##protect##"functionStart_InitAndStartWatchdog"
			returnValue = DllImport.i_PCIe1516_InitAndStartWatchdog(this.handle, reloadValue, timeBase);
//##protect##"functionEnd_InitAndStartWatchdog"
//##protect##"functionEnd_InitAndStartWatchdog"
			return returnValue;
		}

		/// <summary>
		/// Reads the value of the watchdog of the board
		/// </summary>
		/// <param name="deviceHandle">Handle of the board</param>
		/// <param name="value">Current value of the watchdog</param>
		public int ReadWatchdogValue(out UInt16 value)
		{
			int returnValue;
//##protect##"functionStart_ReadWatchdogValue"
//##protect##"functionStart_ReadWatchdogValue"
			returnValue = DllImport.i_PCIe1516_ReadWatchdogValue(this.handle, out value);
//##protect##"functionEnd_ReadWatchdogValue"
//##protect##"functionEnd_ReadWatchdogValue"
			return returnValue;
		}

		/// <summary>
		/// Reads the status of the watchdog of the board
		/// </summary>
		/// <param name="deviceHandle">Handle of the board</param>
		/// <param name="status">Current status of the watchdog (0: disabled, 1: enabled)</param>
		/// <param name="softwareTriggerStatus">Current status of the software trigger (0: no trigger occured, 1: a trigger occured)</param>
		public int ReadWatchdogStatus(out Byte status, out Byte softwareTriggerStatus)
		{
			int returnValue;
//##protect##"functionStart_ReadWatchdogStatus"
//##protect##"functionStart_ReadWatchdogStatus"
			returnValue = DllImport.i_PCIe1516_ReadWatchdogStatus(this.handle, out status, out softwareTriggerStatus);
//##protect##"functionEnd_ReadWatchdogStatus"
//##protect##"functionEnd_ReadWatchdogStatus"
			return returnValue;
		}

		/// <summary>
		/// Triggers the watchdog
		/// </summary>
		/// <param name="deviceHandle">Handle of the board</param>
		public int TriggerWatchdog()
		{
			int returnValue;
//##protect##"functionStart_TriggerWatchdog"
//##protect##"functionStart_TriggerWatchdog"
			returnValue = DllImport.i_PCIe1516_TriggerWatchdog(this.handle);
//##protect##"functionEnd_TriggerWatchdog"
//##protect##"functionEnd_TriggerWatchdog"
			return returnValue;
		}

		/// <summary>
		/// Stops and releases the watchdog of the board
		/// </summary>
		/// <param name="deviceHandle">Handle of the board</param>
		public int StopAndReleaseWatchdog()
		{
			int returnValue;
//##protect##"functionStart_StopAndReleaseWatchdog"
//##protect##"functionStart_StopAndReleaseWatchdog"
			returnValue = DllImport.i_PCIe1516_StopAndReleaseWatchdog(this.handle);
//##protect##"functionEnd_StopAndReleaseWatchdog"
//##protect##"functionEnd_StopAndReleaseWatchdog"
			return returnValue;
		}

#endregion
#region "User"
//##protect##"AssemblyUserCode"
//##protect##"AssemblyUserCode"
#endregion
    }
}
