using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Watchdog
{
    public partial class Watchdog : Form
    {
        /// <summary>
        /// Board object
        /// </summary>
        private PCIe1516_NET.PCIe1516 board;

        /// <summary>
        /// Useful array
        /// </summary>
        private CheckBox[] outputCheckBoxes;

        /// <summary>
        /// Constructor of the form
        /// </summary>
        public Watchdog()
        {
            InitializeComponent();

            // Construct the useful array
            this.outputCheckBoxes = new CheckBox[8];
            this.outputCheckBoxes[0] = this.output0CheckBox;
            this.outputCheckBoxes[1] = this.output1CheckBox;
            this.outputCheckBoxes[2] = this.output2CheckBox;
            this.outputCheckBoxes[3] = this.output3CheckBox;
            this.outputCheckBoxes[4] = this.output4CheckBox;
            this.outputCheckBoxes[5] = this.output5CheckBox;
            this.outputCheckBoxes[6] = this.output6CheckBox;
            this.outputCheckBoxes[7] = this.output7CheckBox;
        }

        /// <summary>
        /// Occurs when the form loads
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Watchdog_Load(object sender, EventArgs e)
        {
            int returnValue = 0;

            // Load the saved settings
            this.timeBaseComboBox.SelectedIndex = Properties.Settings.Default.timeBase;
            this.reloadValueTextBox.Text = Properties.Settings.Default.reloadValue.ToString();

            // Set the output selected index
            this.outputIndexComboBox.SelectedIndex = 0;

            // Open the first board
            this.board = PCIe1516_NET.PCIe1516.OpenBoard(0);
            if (board == null)
            {
                MessageBox.Show("Error while opening board 0");
                this.Close();
                return;
            }

            // Set the board interrupt routine
            if ((returnValue = this.board.SetBoardIntRoutine()) != 0)
            {
                MessageBox.Show("Error while setting interrupt routine. " + returnValue);
                this.Close();
                return;
            }

            // Set digital memory to on
            if ((returnValue = this.board.SetDigitalOutputMemoryOn()) != 0)
            {
                MessageBox.Show("Error while setting digital memory to on. " + returnValue);
                this.Close();
                return;
            }

            // Set all output to on
            if ((returnValue = this.board.Set8DigitalOutputsOn(255)) != 0)
            {
                MessageBox.Show("Error while setting digital outputs to on. " + returnValue);
                this.Close();
                return;
            }
        }

        /// <summary>
        /// Occurs when the form closes
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Watchdog_FormClosing(object sender, FormClosingEventArgs e)
        {
            int returnValue = 0;

            // Set all output to off
            if ((returnValue = this.board.Set8DigitalOutputsOff(255)) != 0)
            {
                MessageBox.Show("Error while setting digital outputs to off. " + returnValue);
                this.Close();
                return;
            }
            
            // Save the settings
            Properties.Settings.Default.timeBase = timeBaseComboBox.SelectedIndex;
            try
            {
                Properties.Settings.Default.reloadValue = Convert.ToUInt16(reloadValueTextBox.Text);
            }
            catch (Exception)
            {
                Properties.Settings.Default.reloadValue = 1000;
            }
            Properties.Settings.Default.Save();

            // Release the watchdog
            board.StopAndReleaseWatchdog();

            // Reset the board interrupt routine
            board.ResetBoardIntRoutine();
        }

        /// <summary>
        /// Click on the start button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void startButton_Click(object sender, EventArgs e)
        {
            Byte timeBase = (Byte)timeBaseComboBox.SelectedIndex;
            UInt16 reloadValue = 0;
            int returnValue = 0;

            // Compute the reload value
            try
            {
                reloadValue = Convert.ToUInt16(reloadValueTextBox.Text);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Invalid reload value, using 1000 instead. " + ex.Message);
                reloadValue = 1000;
                reloadValueTextBox.Text = "1000";
            }

            // Check the reload value
            if ( (reloadValue < 1) || (reloadValue > 4095) )
            {
                MessageBox.Show("Invalid reload value, using 1000 instead.");
                reloadValue = 1000;
                reloadValueTextBox.Text = "1000";
            }

            // Start the watchdog with interrupt
            if ((returnValue = this.board.InitAndStartWatchdog(reloadValue, timeBase)) != 0)
            {
                MessageBox.Show("Error while starting watchdog. " + returnValue);
                return;
            }

            // Enables button
            startButton.Enabled = false;
            triggerButton.Enabled = true;
            stopButton.Enabled = true;

            // Start timer
            displayTimer.Start();
        }

        /// <summary>
        /// Click on the stop button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void stopButton_Click(object sender, EventArgs e)
        {
            int returnValue = 0;

            // Stop the watchdog
            if ((returnValue = this.board.StopAndReleaseWatchdog()) != 0)
            {
                MessageBox.Show("Error while stopping watchdog. " + returnValue);
                return;
            }

            // Enables button
            startButton.Enabled = true;
            triggerButton.Enabled = false;
            stopButton.Enabled = false;

            // Stop timer
            displayTimer.Stop();
        }

        /// <summary>
        /// Click on the trigger button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void triggerButton_Click(object sender, EventArgs e)
        {
            int returnValue = 0;

            // Trigger the watchdog
            if ((returnValue = this.board.TriggerWatchdog()) != 0)
            {
                MessageBox.Show("Error while triggering watchdog. " + returnValue);
                return;
            }
        }

        /// <summary>
        /// Enables to update the values of the output checkboxes
        /// </summary>
        private void UpdateOutputValues()
        {
            Byte outputValues = 0;
            int returnValue = 0;

            // Read the output status
            if ((returnValue = this.board.Get8DigitalOutputsStatus(out outputValues)) != 0)
            {
                MessageBox.Show("Error while reading digital output status. " + returnValue);
                return;
            }

            // Update checkboxes status
            for (int i = 0; i < 8; i++)
            {
                if ((outputValues & (1 << i)) == (1 << i))
                {
                    outputCheckBoxes[i].Checked = true;
                }
                else
                {
                    outputCheckBoxes[i].Checked = false;
                }
            }
        }

        /// <summary>
        /// Click on the on button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void onButton_Click(object sender, EventArgs e)
        {
            int returnValue = 0;

            // Set to on
            if ((returnValue = this.board.Set1DigitalOutputOn((byte)outputIndexComboBox.SelectedIndex)) != 0)
            {
                MessageBox.Show("Error while setting one digital output to on. " + returnValue);
                return;
            }
        }

        /// <summary>
        /// Click on the off button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void offButton_Click(object sender, EventArgs e)
        {
            int returnValue = 0;

            // Set to off
            if ((returnValue = this.board.Set1DigitalOutputOff((byte)outputIndexComboBox.SelectedIndex)) != 0)
            {
                MessageBox.Show("Error while setting one digital output to off. " + returnValue);
                return;
            }
        }

        /// <summary>
        /// Occurs when the timer ticks
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void displayTimer_Tick(object sender, EventArgs e)
        {
            UInt16 watchdogValue = 0;
            int returnValue = 0;

            // Get watchdog value
            if ((returnValue = this.board.ReadWatchdogValue(out watchdogValue)) != 0)
            {
                displayTimer.Stop();

                // Stop
                this.board.StopAndReleaseWatchdog();

                // Enables button
                startButton.Enabled = true;
                stopButton.Enabled = false;
                triggerButton.Enabled = false;

                MessageBox.Show("Error while reading watchdog value. " + returnValue);
                return;
            }

            // Update value
            UpdateOutputValues();

            // Update watchdog value
            watchdogValueLabel.Text = watchdogValue.ToString();
        }
    }
}